// Listing 31.2: Enumerating the contents of a certificate store

using System;
using System.IO;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Runtime.InteropServices;

public class Test 
{
  // magic constants from wincrypt.h
  private static int CERT_STORE_PROV_SYSTEM = 10;
  private static int CERT_SYSTEM_STORE_CURRENT_USER = (1 << 16);

  [DllImport("CRYPT32.DLL", EntryPoint="CertOpenStore", CharSet=CharSet.Auto, SetLastError=true)]
  public static extern IntPtr CertOpenStoreStringPara( int storeProvider, int encodingType, int hcryptProv, int flags, String pvPara);

  [DllImport("CRYPT32.DLL", EntryPoint="CertOpenStore", CharSet=CharSet.Auto, SetLastError=true)]
  public static extern IntPtr CertOpenStoreIntPtrPara( int storeProvider, int encodingType, int hcryptProv, int flags, IntPtr pvPara);

  [DllImport("CRYPT32.DLL", EntryPoint="CertEnumCertificatesInStore", CharSet=CharSet.Auto, SetLastError=true)]
  public static extern IntPtr CertEnumCertificatesInStore(IntPtr storeProvider, IntPtr prevCertContext);

  [DllImport("CRYPT32.DLL", EntryPoint="CertCloseStore", CharSet=CharSet.Auto, SetLastError=true)]
  public static extern bool CertCloseStore(IntPtr storeProvider, int flags);

  public static void Main(String[] args)  
  {
    String store = "MY"; // default system store to search is the MY store
    if (args.Length > 0) 
    {
      store = args[0]; // first argument, if any, overrides default value
    }
    IntPtr storeHandle = CertOpenStoreStringPara(CERT_STORE_PROV_SYSTEM, 0, 0, CERT_SYSTEM_STORE_CURRENT_USER, store);
    X509Certificate cert;
    IntPtr currentCertContext = CertEnumCertificatesInStore(storeHandle, (IntPtr) 0);
    while (currentCertContext != (IntPtr) 0)  
    {
      cert = new X509Certificate(currentCertContext);
      // Do something with the certificate...
      Console.WriteLine(cert.ToString(true));
      currentCertContext = CertEnumCertificatesInStore(storeHandle, currentCertContext);
    }
    CertCloseStore(storeHandle, 0);
  }
}
