// Listing 31.5: Encrypting an on-disk file to a second file with random IV generation

public static void EncryptFile(byte[] key, String sourceFile, String destFile) {
  // initialize with random key and IV
  Rijndael aes = Rijndael.Create();
  // set the key to the passed-in arg
  aes.Key = key;
  // create FileStreams for source and dest
  FileStream inStream = new FileStream(sourceFile, FileMode.Open);
  FileStream outStream = new FileStream(destFile, FileMode.Create);
  // We want to write the IV out to the destFile unencrypted,
  // so in this case we can wrap a CryptoStream around the *sourceFile*
  // and read encrypted bytes from it
  CryptoStream encryptedInStream = new CryptoStream(inStream, aes.CreateEncryptor(), CryptoStreamMode.Read);
  // Write the IV out to the ciphertext file as the first bytes in the file.
  outStream.Write(aes.IV, 0, aes.IV.Length);
  // Now we're ready to encrypt. Read the bytes from the CryptoStream
  // in a loop until there aren't any more (end-of-file), writing to 
  // the output file as we go. We need a buffer to hold what we're going to 
  // read, and bytesRead tells us how many bytes in buffer are 
  //valid ciphertext.
  int bytesRead;
  byte[] buffer = new byte[1024]; // read 1K at a time
  do {
      bytesRead = encryptedInStream.Read(buffer,0,1024);
      outStream.Write(buffer, 0, bytesRead);
  } while (bytesRead > 0);   
  // Done!  Close() the streams we used.
  inStream.Close();
  outStream.Close();
  // Clear the Rijndael and CryptoStream objects to zero out
  // sensitive data (e.g. key and IV in Rijndael)
  encryptedInStream.Clear();
  aes.Clear();
  // Clearing the buffer is not strictly required since it only holds
  // ciphertext, but is good practice also.
  Array.Clear(buffer,0,buffer.Length);
  return;
}
